const Veritabanı = require('better-sqlite3');
const util = require('util');
let db;

if (!db) db = new Veritabanı(`./somon.sqlite`);

var yöntemler = {
  çek: require('../kitaplık/çek.js'),
  kur: require('../kitaplık/kur.js'),
  ekle: require('../kitaplık/ekle.js'),
  çıkart: require('../kitaplık/çıkart.js'),
  ittir: require('../kitaplık/ittir.js'),
  sil: require('../kitaplık/sil.js'),
  kontrol: require('../kitaplık/kontrol.js'),
  hepsiniÇek: require('../kitaplık/hepsiniÇek.js'),
  tip: require('../kitaplık/tip.js')
};

module.exports = { 
  
  versiyon: 'v1.0.0-somon',
  
  çek: function(key, ops) {
    if (!key) throw new TypeError('Veritabanında araştırmam için bir isim girmelisiniz.');
    return somon('çek', {id: key, ops: ops || {}});
  },
  
  kur: function(key, value, ops) {
    if (!key) throw new TypeError('Veritabanında araştırmam için bir isim girmelisiniz.');
    if (value === undefined) throw new TypeError('Veritabanına kaydetmem için bir değer belirtmelisiniz.');
    return somon('kur', {stringify: true, id: key, data: value, ops: ops || {}});
  },
  
  arttır: function(key, value, ops) {
    if (!key) throw new TypeError('Veritabanında araştırmam için bir isim girmelisiniz.');
    if (isNaN(value)) throw new TypeError('Veritabanına arttırmam için bir sayı belirtmelisiniz.');
    return somon('ekle', {id: key, data: value, ops: ops || {}});
  },
  
  azalt: function(key, value, ops) {
    if (!key) throw new TypeError('Veritabanında araştırmam için bir isim girmelisiniz.');
    if (isNaN(value)) throw new TypeError('Veritabanına azaltmam için bir sayı belirtmelisiniz.');
    return somon('çıkart', {id: key, data: value, ops: ops || {}});
  },
  
  ekle: function(key, value, ops) {
    if (!key) throw new TypeError('Veritabanında araştırmam için bir isim girmelisiniz.');
    if (isNaN(value)) throw new TypeError('Veritabanına arttırmam için bir sayı belirtmelisiniz.');
    return somon('ekle', {id: key, data: value, ops: ops || {}});
  },
  
  çıkart: function(key, value, ops) {
    if (!key) throw new TypeError('Veritabanında araştırmam için bir isim girmelisiniz.');
    if (isNaN(value)) throw new TypeError('Veritabanına azaltmam için bir sayı belirtmelisiniz.');
    return somon('çıkart', {id: key, data: value, ops: ops || {}});
  },

  ittir: function(key, value, ops) {
    if (!key) throw new TypeError('Veritabanında araştırmam için bir isim girmelisiniz.');
    if (!value && value != 0) throw new TypeError('Veritabanına ittirmem için değer belirtmelisiniz.');
    return somon('ittir', {stringify: true, id: key, data: value, ops: ops || {}});
  },
  
  sil: function(key, ops) {
    if (!key) throw new TypeError('Veritabanında araştırmam için bir isim girmelisiniz.');
    return somon('sil', {id: key, ops: ops || {}});
  },
  
  varyok: function(key, ops) {
    if (!key) throw new TypeError('Veritabanında kontrol etmem için bir isim girmelisiniz.');
    return somon('kontrol', {id: key,  ops: ops || {}});
  },
  
  kontrol: function(key, ops) {
    if (!key) throw new TypeError('Veritabanında kontrol etmem için bir isim girmelisiniz.');
    return somon('kontrol', {id: key,  ops: ops || {}});
  },
  
  all: function(ops) { 
    return somon('hepsiniÇek', {ops: ops || {}});
  },
  
  fetchAll: function(ops) { 
    return somon('hepsiniÇek', {ops: ops || {}});
  },
  
  type: function(key, ops) {
    if (!key) throw new TypeError('Veritabanında araştırmam için bir isim girmelisiniz.');
    return somon('tip', {id: key, ops: ops || {}});
  },
  
}

function somon(yöntem, args, tableName) {
  
  let ayarlar = {
    table: tableName || args.ops.table || 'json'
  }
  
  db.prepare(`CREATE TABLE IF NOT EXISTS ${ayarlar.table} (ID TEXT, json TEXT)`).run();

  if (args.ops.target && args.ops.target[0] === '.') args.ops.target = args.ops.target.slice(1); // Remove prefix if necessary
  if (args.data && args.data === Infinity) throw new TypeError(`You cannot set Infinity into the database @ ID: ${args.id}`)
  
  if (args.stringify) {
    try { args.data = JSON.stringify(args.data); } catch (e) 
    { throw new TypeError(`Please supply a valid input @ ID: ${args.id}\nError: ${e.message}`); } 
  }
  
  // Translate dot notation from keys
  if (args.id && args.id.includes('.')) {
    let unparsed = args.id.split('.');
    args.id = unparsed.shift();
    args.ops.target = unparsed.join('.');
  }
  
  return yöntemler[yöntem](db, args, ayarlar);
  
}
